import { world, system } from "@minecraft/server";

// Define an array of block type IDs
const saplingTypes = [
    "minecraft:oak_sapling",
    "minecraft:birch_sapling",
    "minecraft:spruce_sapling",
    "minecraft:jungle_sapling",
    "minecraft:acacia_sapling",
    "minecraft:dark_oak_sapling",
    "minecraft:pale_oak_sapling",
    "minecraft:wheat",
    "minecraft:carrots",
    "minecraft:beetroot",
    "minecraft:potatoes",
    "minecraft:reeds",
    "minecraft:bamboo_sapling",
    "minecraft:bamboo",
    "minecraft:sweet_berry_bush"
];

// Helper function to check if a block matches saplingTypes
function isSaplingType(block) {
    return saplingTypes.includes(block.typeId);
}

system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const sneaking = player.isSneaking;
        const hasTag = player.hasTag("sneaking");

        if (sneaking && !hasTag) {
            // Add "sneaking" tag to prevent duplicate processing
            player.addTag("sneaking");

            const randomScore = Math.floor(Math.random() * (10 - 5 + 1)) + 5; // Random score between 1 and 25
            player.runCommand(`scoreboard players add @e[type=sneaking_grower:growing_checker,r=5] grower ${randomScore}`);

            // Get player's location and dimension
            const playerPos = player.location;
            const dimension = player.dimension;

            // Collect sapling positions (ignoring Y)
            const saplingPositions = [];

            // Loop through a 5x5 area around the player (ignore Y)
            for (let dx = -3; dx <= 3; dx++) {
                for (let dz = -3; dz <= 3; dz++) {
                    const checkPos = {
                        x: Math.floor(playerPos.x + dx),
                        y: Math.floor(playerPos.y), // Use player's Y (no change in Y)
                        z: Math.floor(playerPos.z + dz),
                    };
                    const block = dimension.getBlock(checkPos);

                    if (block && isSaplingType(block)) {
                        const checkCommand = `execute positioned ${checkPos.x} ${checkPos.y + 0.5} ${checkPos.z} as @e[type=sneaking_grower:growing_checker,r=0.001] if entity @e[type=sneaking_grower:growing_checker,c=2,score={grower=..1},r=0.001] run kill @s`;
                        try {
                            const result = dimension.runCommand(checkCommand);

                            // Add to the sapling list only if no entity exists
                            if (!result.successCount) {
                                saplingPositions.push(checkPos);
                            }
                        } catch {
                            saplingPositions.push(checkPos);
                        }
                    }
                }
            }

            // Summon all entities simultaneously and ensure only 1 entity per block
            for (const pos of saplingPositions) {
                const summonCommand = `summon sneaking_grower:growing_checker ${pos.x} ${pos.y + 0.5} ${pos.z}`;
                dimension.runCommand(summonCommand);

                // Ensure only one entity remains per block
                const removeCommand = `execute positioned ${pos.x} ${pos.y + 0.5} ${pos.z} as @e[type=sneaking_grower:growing_checker,r=0.1] if entity @e[type=sneaking_grower:growing_checker,c=2,r=0.1,scores={grower=..1}] run kill @e[type=sneaking_grower:growing_checker,scores={grower=..1},r=0.01]`;
                dimension.runCommand(removeCommand);
            }
        } else if (!sneaking && hasTag) {
            // Remove the "sneaking" tag when the player stops sneaking
            player.removeTag("sneaking");
        }
    }
}, 1);